// <html><head><title> MpxG2Structs.h </title></head><body bgcolor=#FFFFFF><pre>
///////////////////////////////////////////////////////////////////////////////
//	Copyright (C) 2000 by Lexicon, Incorporated, Bedford, Massachusetts, USA
//	All Rights Reserved, Worldwide
//
// Under the copyright laws this program may not be copied, in whole or in part, 
// without the express written consent of Lexicon, Incorporated.  Under the law, 
// copying includes translating into another language or format.
//
// This program is protected as an unpublished work under the copyright laws of 
// The United States of America.  The above copyright notice is not intended to 
// constitute a publication of this work.
//
////////////////////////////////////////////////////////////////////////////////
//	    Project: MPX G2
//     Filename: MpxG2Structs.h
//	Description: This file contains structures used in the MPX G2
//
//	Revision History:
//------------------------------------------------------------------------------
//	12-29-97	rjs		Created.
//	06-01-00	rjs		Prepped for public distribution.
//	
//////////////////////////////////////////////////////////////////////////////


///////////////////////////////
// R1 Remote Interface Struct

// HTML Tag: <A NAME="REMOTE_UPDATE_DATA"></A>
typedef struct
{
	UINT8 cc_num[9];	// fx1,fx2,chorus,delay,reverb,eq,gain,insert,bypass
	UINT16 state_bitmap;	// bitmap of on/off state of the blocks
		/*	bit 0:	fx1			bit 8:	bypass
			bit 1:	fx2			bit 9:	a/b
			bit 2:	chorus
			bit 3:	delay
			bit 4:	reverb
			bit 5:	eq
			bit 6:	gain
			bit 7:	insert
		*/
	UINT16 available_bitmap;	//	bitmap of availability of states
		/*	bit 0:	fx1
			bit 1:	fx2
			bit 2:	chorus
			bit 3:	delay
			bit 4:	reverb
			bit 5:	eq
			bit 6:	gain
			bit 7:	insert
		*/
	UINT16 prog_num;
} REMOTE_UPDATE_DATA;


// HTML Tag: <A NAME="MENU_EVENT"></A>
////////////////////////////
typedef struct
{
	UINT8 	type;
	UINT8	value;
	UINT16	time;
} MENU_EVENT;


////////////////////////////
// System "Setup" structure

// HTML Tag: <A NAME="SETUP"></A>
typedef struct
{
	UINT8		midi_in_channel;
	UINT16		midi_output_rate;
	UINT8		midi_device_id;
	UINT8		remote_device_id;
	UINT8		midi_soft_thru;
	UINT8		sysex_on_off;
	UINT8		midi_out_channel;
	UINT16		program_change_offset;
	UINT8		midi_automation_mode;
	UINT8		midi_clock_mode;
	UINT8 		sort_mode;	
	UINT8 		cur_contrast;
	UINT8		automation_device_id;
	UINT8		program_change_mode;
	UINT8		cont_smooth;
	UINT8		memory_protect_mode;
	UINT8		auto_store_mode;
	UINT8		sleep_mode;
	UINT8		mix_mode;
	UINT8		program_load_bypass_mode;
	UINT8		pgm_inc_controller;
	UINT8		pgm_dec_controller;
	UINT8		soft_saturation_mode;
	UINT8  		pedal_max;
	UINT8  		pedal_delta;
	UINT8		tempo_mode;
	UINT8		autoload_mode;
	UINT8		send_bypass_mode;
	UINT8 		post_bypass_mode;
	UINT8		insert_mode;
	UINT8		global_spkr_sim_mode;
	UINT8		global_noise_gate_mode;
	UINT8		global_reverb_mode;
	UINT8		global_bypass_mode;
	UINT8		tuner_cal;
	INT8 		tuner_offset;
	UINT8		tuner_bypass_mode;
	UINT8		output_mode;
	UINT8		bypass_patch_sources[NUM_BYPASS_PATCHES];
	char 		setup_name[SETUP_NAME_SIZE];
}SETUP;


// HTML Tag: <A NAME="CHOICES"></A>
///////////////////////////////
// System "Choices" structure
typedef struct
{
	UINT8		program_bank;	
	UINT8 		current_effect_edit_parameter;
	UINT8 		cur_alg_select;
	UINT8 		cur_internal_cont_param_num;
	UINT8 		cur_internal_cont;
	UINT8 		compare_mode;
	UINT8 		dbase_edit_cur_choice;
	UINT8 		soft_row;
	UINT8 		soft_row_edit;
	UINT8 		current_map_block;
	UINT8 		cur_routing_item;
	UINT8		cur_map_screen;
	UINT8 		order_cur_choice;
	UINT8 		mix_cur_choice;
	UINT8 		audio_menu;
	UINT8 		audio_options_cur_choice;
	UINT8 		midi_menu;
	UINT8 		chains_menu;
	UINT8 		chain;
	UINT8 		chain_link;
	UINT8		midi_map_menu;
	UINT8		midi_map;
	UINT8		midi_map_element;
	UINT8 		store_cur_choice;
	UINT8 		cur_edit_select_choice;
	UINT8 		cur_system_select_choice;
	UINT8 		cur_edit_page;
	char 		*cur_edit_page_name_ptr;
	UINT8 		cur_edit_param_index;
	UINT8 		cur_num_edit_params;
	UINT8 		cur_system_page;
	char 		*cur_system_page_name_ptr;
	UINT8 		cur_system_param_index;
	UINT8 		cur_num_system_params;
	UINT16		selected_program_num;
	UINT16		running_program_num;
	UINT8		operating_mode;
	UINT8		button_encoder_mode;
	UINT8		cur_setup_num;
	UINT8		setup_menu;
	UINT8		cur_utility_choice;
	UINT8 		bypass_patch_menu;
	UINT8		global_patch;
	UINT8		global_patch_menu;
	UINT8 		cur_ui_dump_type;
	UINT8		cont_remap;
	UINT8		patch_menu;
	UINT8 		cur_patch_num;
	UINT8		patch_knob_mode;
	UINT8		cur_auto_store_choice;
	UINT8		tempo_menu;
	UINT8		cur_tempo_source_error_choice;
	UINT8		cur_verify_choice;
	UINT8		cur_verify_store_choice;
	UINT8		cur_prog_sel_choice;
	UINT8		cur_sleep_choice;
}CHOICES;

// HTML Tag: <A NAME="CONTROL_ADDRESS"></A>
/////////////////////////////////////////////////////////
// LUSP Control Address

// Internally, the MPX G2 uses 8 bit values to store the
// control address.
typedef struct
{
	UINT8 num_levels;
	UINT8 level_A;
	UINT8 level_B;
	UINT8 level_C;
	UINT8 level_D;
	UINT8 level_E;
}CONTROL_ADDRESS;

// HTML Tag: <A NAME="LARGE_CONTROL_ADDRESS"></A>
// Externally, (via MIDI) 16 bit values are used
typedef struct
{
	UINT16 num_levels;
	UINT16 level_A;
	UINT16 level_B;
	UINT16 level_C;
	UINT16 level_D;
	UINT16 level_E;
}LARGE_CONTROL_ADDRESS;


////////////////////////////////////////////////////
// Program Structures
////////////////////////////////////////////////////

// HTML Tag: <A NAME="PARAM_DATA"></A>
typedef struct
{
	UINT8 data[NUM_EFFECT_TYPES][MAX_PARAM_BYTES];
}PARAM_DATA;


///////////////////////
// Patch Structures

// HTML Tag: <A NAME="GLOBAL_PATCHES"></A>
typedef struct
{
	UINT8 sources[NUM_GLOBAL_PATCHES];
	UINT8 dests[NUM_GLOBAL_PATCHES];
}GLOBAL_PATCHES;

// HTML Tag: <A NAME="PATCH_SOURCE"></A>
typedef struct
{
	UINT8 	cont_num;
	UINT8 	min;
	UINT8 	mid;
	UINT8 	max;
}PATCH_SOURCE;

// HTML Tag: <A NAME="PATCH_DEST"></A>
typedef struct
{
	UINT8 	effect_type;
	UINT8 	param_index;
	UINT16 	min;
	UINT16 	mid;
	UINT16 	max;
}PATCH_DEST;

// HTML Tag: <A NAME="PATCH"></A>
typedef struct
{
	PATCH_SOURCE source;
	PATCH_DEST	 dest;
}PATCH;


//////////////////////////////
// Effect Routing Structures

// HTML Tag: <A NAME="EFFECT_BLOCK"></A>
typedef struct
{
	UINT8	effect;
	UINT8	upper_input_conn;
	UINT8	lower_input_conn;
	UINT8	routing;
	UINT8	path_type;
} EFFECT_BLOCK;

// HTML Tag: <A NAME="TOE_PATCHES"></A>
typedef struct
{
	UINT8	fx_1;
	UINT8	fx_2;
	UINT8 	chorus;
	UINT8	delay;
	UINT8	reverb;
	UINT8	eq;
	UINT8	gain;
} TOE_PATCHES;


//////////////////////////////////////
// Internal Control Source Structures

// HTML Tag: <A NAME="KNOB_DATA"></A>
typedef struct 
{
	UINT8 		value;
	UINT8 		min;
	UINT8 		max;
	char		name[KNOB_NAME_SIZE];
}KNOB_DATA;


// HTML Tag: <A NAME="LFO_DATA"></A>
typedef struct 
{
	UINT8  		mode;
	UINT16 		rate;	
	UINT8		rate_units;
	UINT8  		skew;	/* pulse width */
	UINT8  		phase;
	UINT8  		gain;	/* depth */
	UINT8  		onLvl;
	UINT8  		onSrc;
} LFO_DATA;

// HTML Tag: <A NAME="RANDOM_GEN_DATA"></A>
typedef struct 
{
	UINT8  		min;
	UINT8  		max;
	UINT16 		rate;
	UINT8		rate_units;
} RANDOM_GEN_DATA;

// HTML Tag: <A NAME="AB_GEN_DATA"></A>
typedef struct 
{
	UINT8  		mode;
	UINT8  		rate;
	UINT8  		inertia;
	UINT8  		onLvl;
	UINT8  		onSrc;
}AB_GEN_DATA;

// HTML Tag: <A NAME="ENVELOPE_DATA"></A>
typedef struct 
{
	UINT8  		source_1;
	UINT8  		source_2;
	UINT8  		attack_rate;
	UINT8  		decay_rate;
}ENVELOPE_DATA;

////////////////////////
// Misc Program elements
// HTML Tag: <A NAME="SPEAKER_SIM"></A>
typedef struct 
{
	UINT8		mode;
	UINT8		cabinet;
}SPEAKER_SIM;

// HTML Tag: <A NAME="NOISE_GATE"></A>
typedef struct 
{
	UINT8		enable;
	UINT8		threshold;
	UINT8		offset;
	UINT16		hold_time;
	UINT16		attack_time;
	UINT16		release_time;
	UINT8		attenuation;
	UINT8		delay_time;
	UINT8		mode;
}NOISE_GATE;


// HTML Tag: <A NAME="TEMPO_DATA"></A>
typedef struct 
{
	UINT16			tempo;
	UINT8			tempo_source;
	UINT8			beat_value;
	UINT8			tap_source;
	UINT8 			tap_average;
	UINT8			tap_source_level;
} TEMPO_DATA;	


// HTML Tag: <A NAME="POST_DATA"></A>
typedef struct 
{
	UINT8			level;
	UINT8			bypass_level;
	UINT8			mix;
} POST_DATA;	

// HTML Tag: <A NAME="SEND_DATA"></A>
typedef struct 
{
	UINT8			level;
	UINT8			bypass_level;
	UINT8			select;
} SEND_DATA;


// HTML Tag: <A NAME="SOFT_ROW"></A>
typedef struct 
{
	UINT8	effect_type;
	UINT8	param_index;
}SOFT_ROW;


// HTML Tag: <A NAME="SORT_FLAGS"></A>
typedef struct
{
	UINT16		FX_type_flags;
	UINT8		guitar_style_flags;
}SORT_FLAGS;

// HTML Tag: <A NAME="SYSTEM_SORT_FLAGS"></A>
typedef struct
{
	SORT_FLAGS	sort_flags[NUM_PROGRAMS];
}SYSTEM_SORT_FLAGS;

// HTML Tag: <A NAME="MPXG2_PROGRAM"></A>
typedef struct 
{					
	PARAM_DATA 		param_data;
	SORT_FLAGS 		sort_flags;
	EFFECT_BLOCK 	routing_blocks[NUM_CONFIGURABLE_BLOCKS];
	TOE_PATCHES 	toe_patches;

	UINT8			alg_nums[NUM_EFFECT_TYPES];
	char			name[PROG_NAME_SIZE];
	UINT8			effects_status; /* bypassed? bitmapped */
	SOFT_ROW		soft_rows[NUM_SOFT_ROW_PARAMS];
	TEMPO_DATA		tempo_data;			
	UINT8			relay_chan;
	PATCH			patches[NUM_PATCHES_PER_PROGRAM];
	KNOB_DATA		knob;
	LFO_DATA		lfo_1;
	LFO_DATA		lfo_2;
	RANDOM_GEN_DATA	random_gen;
	AB_GEN_DATA		ab_gen;
	ENVELOPE_DATA	envelope;
	NOISE_GATE		noise_gate;
	UINT8			bypass_state;
	SPEAKER_SIM		speaker_sim;
	POST_DATA		post;
	SEND_DATA		send;
}MPXG2_PROGRAM;


// </pre></body></html>